/*  ________________________________________________________________________  */
/* |----------------------  P R O J E C T   N O T E S  ---------------------| */
/* |________________________________________________________________________| */
/* |                                                                        | */
/* |                           frydaysoft.de                                | */
/* |                                                                        | */
/* |________________________________________________________________________| */
/* | author     :   Sven Freitag                                            | */
/* |________________________________________________________________________| */
/* |****************************  S O U R C E  *****************************| */
/* |________________________________________________________________________| */
/* | project    :   LightCtrl                                               | */
/* |________________________________________________________________________| */
/* | file name  :   SoftTimer.c      date: 12.11.2009      version :  1.0   | */
/* |________________________________________________________________________| */
/* | task       :   Software Timer module                                   | */
/* |________________________________________________________________________| */
/* | target     :   ATMEL ATmega48/88, 3.6864 MHz                           | */
/* |________________________________________________________________________| */
/* | compiler   :   WIN AVR C Compiler                                      | */
/* |________________________________________________________________________| */
/* | history    :   01.09.2009  project start                               | */
/* |________________________________________________________________________| */
/* | tested by  :                                                           | */
/* |________________________________________________________________________| */


// *****************************************************************************
// ******************  Compiler and Preprocessor Directives  *******************
// *****************************************************************************


// *****************************************************************************
// ***************************  Include Directives  ****************************
// *****************************************************************************

#include "Global.h"
#include "SoftTimer.h"
#include "SoftTimer.def"


// *****************************************************************************
// ******************  Module Global Definitions and Macros  *******************
// *****************************************************************************


// *****************************************************************************
// ********************  Module Global Variable Definitions  *******************
// *****************************************************************************

volatile UCHAR   uc_time_1ms;


// *****************************************************************************
// ********************  Module Local Function Prototypes  *********************
// *****************************************************************************


// *****************************************************************************
// *****************************  Global Functions  ****************************
// *****************************************************************************

// *****************************************************************************
// ***                                SoftTimerInit()                        ***
// *****************************************************************************
// ***  Initializes Timer2 for 1ms and enables compare match A interrupt.    ***
// *****************************************************************************
// ***  Function parameters:                                                 ***
// ***                                    void                               ***
// *****************************************************************************
// ***  Function result:                                                     ***
// ***                                    void                               ***
// *****************************************************************************

void SoftTimerInit(void)
{ 
  // ***********  Init Timer 2 to CTC mode with 1 ms resolution  ***************
  TIMER2_COUNTER_REG = 0x00;            // clear timer2 counter register
  TIMER2_INIT(DIV_1MS);                 // set default 1 ms output compare interrupt value

  uc_time_1ms = 0;                      // clear all variables

  TIMER2_SET_TCCR2A_CTC(TIMER2_TCCR2A_INIT);
  TIMER2_SET_TCCR2B_CTC(TIMER2_TCCR2B_INIT);
  ENABLE_TIMER2_COMP_MATCH_A_INT();
}


// *****************************************************************************
// ***                               SoftTimerDeInit()                       ***
// *****************************************************************************
// ***  DeInitializes Timer2.                                                ***
// *****************************************************************************
// ***  Function parameters:                                                 ***
// ***                                    void                               ***
// *****************************************************************************
// ***  Function result:                                                     ***
// ***                                    void                               ***
// *****************************************************************************

void SoftTimerDeInit(void)
{
  // *******************************  DeInit Timer 2 ***************************
  DISABLE_TIMER2_COMP_MATCH_A_INT();
  TIMER2_SET_TCCR2A_CTC(TIMER2_DEINIT_VALUE);
  TIMER2_SET_TCCR2B_CTC(TIMER2_DEINIT_VALUE);
}


// *****************************************************************************
// ***                             ucSoftTimerGetTime1ms()                   ***
// *****************************************************************************
// ***  Function to get the current timecounter value (resolution = 1 ms).   ***
// *****************************************************************************
// ***  Function parameters:                                                 ***
// ***                                    void                               ***
// *****************************************************************************
// ***  Function result:                                                     ***
// ***    UCHAR                 : current timecounter (resolution = 1 ms)    ***
// *****************************************************************************

UCHAR ucSoftTimerGetTime1ms(void)
{
  UCHAR ucResult;
  
  // ******************  read current counter value  ***************************
  
  DISABLE_TIMER2_COMP_MATCH_A_INT();
  ucResult = uc_time_1ms;
  ENABLE_TIMER2_COMP_MATCH_A_INT();
  
  return(ucResult);
}


// *****************************************************************************
// ***                          ucSoftTimerGetGetTimeDiff1ms()               ***
// *****************************************************************************
// ***  Function calculates the difference time (in ms) between the          ***
// ***  'start time' and the current time.                                   ***
// *****************************************************************************
// ***  Function parameters:                                                 ***
// ***    UCHAR uc_start_time_mark  :  start time                            ***
// *****************************************************************************
// ***  Function result:                                                     ***
// ***    UCHAR                     :  difference time in 1ms.               ***
// *****************************************************************************

UCHAR ucSoftTimerGetGetTimeDiff1ms(UCHAR uc_start_time_mark)
{
  UCHAR uc_result;
  
  // **************  read and calculate  difference value  *********************
  
  DISABLE_TIMER2_COMP_MATCH_A_INT();
  uc_result = ( uc_time_1ms - uc_start_time_mark);
  ENABLE_TIMER2_COMP_MATCH_A_INT();
  
  return(uc_result);
}



// *****************************************************************************
// *****************************  Interrupt Routine  ***************************
// *****************************************************************************

// *****************************************************************************
// ***                          Timer2CompMatchAInt()                        ***
// *****************************************************************************
// ***  Timer2 compare match A interrupt service routine.                    ***
// *****************************************************************************
// ***  Function parameters:                                                 ***
// ***    void                                                               ***
// *****************************************************************************
// ***  Function result:                                                     ***
// ***    void                                                               ***
// *****************************************************************************
ISR(TIMER2_COMPA_vect)
//############################################################################
{  
  // ****************  update 1ms and 100ms softtimer counter  *****************
  uc_time_1ms++;
}


// ********************************  End of File *******************************

