/*  ________________________________________________________________________  */
/* |----------------------  P R O J E C T   N O T E S  ---------------------| */
/* |________________________________________________________________________| */
/* |                                                                        | */
/* |                           frydaysoft.de                                | */
/* |                                                                        | */
/* |________________________________________________________________________| */
/* | author     :   Sven Freitag                                            | */
/* |________________________________________________________________________| */
/* |****************************  S O U R C E  *****************************| */
/* |________________________________________________________________________| */
/* | project    :   LightCtrl                                               | */
/* |________________________________________________________________________| */
/* | file name  :   Uart.c           date: 12.11.2009      version :  1.0   | */
/* |________________________________________________________________________| */
/* | task       :   Uart module                                             | */
/* |________________________________________________________________________| */
/* | target     :   ATMEL ATmega48/88, 3.6864 MHz                           | */
/* |________________________________________________________________________| */
/* | compiler   :   WIN AVR C Compiler                                      | */
/* |________________________________________________________________________| */
/* | history    :   01.09.2009  project start                               | */
/* |________________________________________________________________________| */
/* | tested by  :                                                           | */
/* |________________________________________________________________________| */

// *****************************************************************************
// ******************  Compiler and Preprocessor Directives  *******************
// *****************************************************************************


// *****************************************************************************
// ***************************  Include Directives  ****************************
// *****************************************************************************

#include "global.h"
#include "Uart.h"


// *****************************************************************************
// ******************  Module Global Definitions and Macros  *******************
// *****************************************************************************

#define UART_DISABLE_TX_DATA_REG_EMPTY_INT()  UCSR0B &= ~(1<<UDRIE0)  // disable tx irq
#define UART_ENABLE_TX_DATA_REG_EMPTY_INT()   UCSR0B |= (1<<UDRIE0)   // enable tx irq
#define UART_DISABLE_RX_INT()                 UCSR0B &= ~(1<<RXCIE0)  // enable rx irq
#define UART_ENABLE_RX_INT()                  UCSR0B |= (1<<RXCIE0)   // enable rx irq


typedef enum {
    RX_IDLE  = 0,   // wait for start token
    RX_LEN   = 1,   // wait for length byte
    RX_DATA  = 2,   // wait for all data point out by length byte
    RX_STOP  = 3,   // wait for end token
    RX_READY = 4,   // block finish
    RX_ERROR = 5,   // reserved state to catch RX errors
} T_eRxState;



// *****************************************************************************
// ********************  Module Global Variable Definitions  *******************
// *****************************************************************************

UCHAR m_auc_rx_buf[RX_MAX_LEN];    // rx buffer for state machine
UCHAR m_rx_pos = 0;                // holds number of reveived data
UCHAR m_rx_exp = 0;                // holds number of expected user data after block sync
T_eRxState e_rx_state = RX_IDLE;   // statemachine control
UCHAR *m_puc_tx_buf = 0;           // pointer for TX
UCHAR m_tx_len = 0;                // TX control (number of untransmitted byte)



// *****************************************************************************
// ********************  Module Local Function Prototypes  *********************
// *****************************************************************************


// *****************************************************************************
// *****************************  Global Functions  ****************************
// *****************************************************************************



/******************************************************************************/
/***                     UartInit()                                         ***/
/******************************************************************************/
/***  Initializes Hardware Uart                                             ***/
/******************************************************************************/
/***  Function parameters:                                                  ***/
/***    ui_baudrate :  baudrate                                             ***/
/******************************************************************************/
/***  Function result:                                                      ***/
/******************************************************************************/

void UartInit(UINT ui_baudrate)
{
  // *** initialize internal members ***
  m_rx_pos = 0;
  m_rx_exp = 0;
  e_rx_state = RX_IDLE;
  m_tx_len = 0;
  m_puc_tx_buf = NULLPTR;
  
  // *** Set Baudrate ***
  UBRR0H = (unsigned char)(ui_baudrate >> 8);
  UBRR0L = (unsigned char)ui_baudrate;

  // *** Enable receiver / transmitter ***
  UCSR0B = (1<<RXEN0)|(1<<TXEN0);  
  
  // *** Set frame format: 8data,no Parity, 1 stop bit ***
  UCSR0C = 6;//(3<<UCSZ01);  

  // *** enable RX interrupt of UART ***
  UART_ENABLE_RX_INT();
}



/******************************************************************************/
/***                   bUartIsTxIdle()                                      ***/
/******************************************************************************/
/***  Test if uart module ready for transmit new data                       ***/
/******************************************************************************/
/***  Function parameters:                                                  ***/
/******************************************************************************/
/***  Function result:                                                      ***/
/***    BOOL                 TRUE: ok          FALSE: error                 ***/
/******************************************************************************/

BOOL bUartIsTxIdle(void)
{
    if (!m_tx_len)
    {
      return TRUE;
    }
    return FALSE;
}



/******************************************************************************/
/***                      bUartSendBlock()                                  ***/
/******************************************************************************/
/***  Send block over Uart with special framing                             ***/
/******************************************************************************/
/***  Function parameters:                                                  ***/
/***    puc_buf : tx buffer                                                 ***/
/***    uc_len :  lenght of tx buffer (0..63)                               ***/
/******************************************************************************/
/***  Function result:                                                      ***/
/***    BOOL                 TRUE: ok          FALSE: error                 ***/
/******************************************************************************/

BOOL bUartSendBlock( UCHAR *puc_buf, UCHAR uc_len)
{
  if ( !m_tx_len )
  {
    m_puc_tx_buf = puc_buf;                                // send byte
    m_tx_len = uc_len;
    UART_ENABLE_TX_DATA_REG_EMPTY_INT();
    return TRUE;
  }
  return FALSE;
}



/******************************************************************************/
/***                        bUartExistRxData()                              ***/
/******************************************************************************/
/***  Test if complete block received                                       ***/
/******************************************************************************/
/***  Function parameters:                                                  ***/
/******************************************************************************/
/***  Function result:                                                      ***/
/***    BOOL                 TRUE: ok          FALSE: no                    ***/
/******************************************************************************/

BOOL bUartExistRxData(void)
{
  T_eRxState x;
  x = e_rx_state;
  if ( x == RX_READY )
  {
    return TRUE;
  }
  return FALSE;
}


/******************************************************************************/
/***                        ucUartCopyRxData()                              ***/
/******************************************************************************/
/***  Copy received data into given buffer and restart receiving            ***/
/******************************************************************************/
/***  Function parameters:                                                  ***/
/***    puc_buf : destination buffer                                        ***/
/******************************************************************************/
/***  Function result:                                                      ***/
/***    copied length                                                       ***/
/******************************************************************************/

UCHAR ucUartCopyRxData( UCHAR *puc_buf)
{
  UCHAR x;

  if ( e_rx_state == RX_READY )
  {
      for (x=0; m_rx_pos; m_rx_pos--)
      {
         *puc_buf++ = m_auc_rx_buf[x++];
      }
      e_rx_state = RX_IDLE;
      UART_ENABLE_RX_INT();
      return x;
  }
  return 0;   
}



// *****************************************************************************
// ***                           Interrupt Routines                          ***
// *****************************************************************************

// *****************************************************************************
// ***                           Uart_Rx_Interrupt()                         ***
// *****************************************************************************
// ***  UART receive interrupt.                                              ***
// *****************************************************************************
// ***  Function parameters:                                                 ***
// *****************************************************************************
// ***  Function result:                                                     ***
// *****************************************************************************
ISR(USART_RX_vect)
{
  UCHAR uc_byte;

  // read error reg --> not used
  uc_byte = UDR0;  
  if ( uc_byte == START_TOK )
  {   // in sync
      e_rx_state = RX_IDLE;
  }
    // simple RX block statemachine
    // receives a block with syntax eg: ">9C12345678"
    // '>': start token for block synchronisation
    // byte after '>': lenght of following data
    switch ( e_rx_state )
    {
        case RX_IDLE:
            if ( uc_byte == START_TOK )
            {   // in sync
                m_auc_rx_buf[0] = uc_byte;
                m_rx_pos = 1;
                m_rx_exp = 0;
                e_rx_state = RX_LEN;
            }
            break;
        case RX_LEN:
            m_auc_rx_buf[1] = uc_byte;
            if ( uc_byte > '9' )
            {   //'a'(61h)-'f' or 'A'(41h)-'F' --> a-f
                uc_byte |= 0x20;    // makeLower()
                uc_byte -= 0x57;    // 'a'-'f' --> 10..15 
            }
            else
            {   
                uc_byte -= '0';     // '0'-'9' --> 0-9
            }
            m_rx_exp = (uc_byte+RX_CMD_HEADER);
            if ( uc_byte > RX_MAX_DATALEN )
            {   // invalid length, try new sync
                e_rx_state = RX_IDLE;
            }
            else
            {   // length valid, wait for user data
                m_rx_pos = 2;
                e_rx_state = RX_DATA;
            }
            break;
        case RX_DATA:               
            if (m_rx_pos < (RX_MAX_LEN-1))
            {   // copy user data
                m_auc_rx_buf[m_rx_pos] = uc_byte;
                if ( (++m_rx_pos) >= m_rx_exp )
                {   // block seems to be valid and complete
                    e_rx_state = RX_STOP;
                }
            }
            else
            {   // buffer overflow, try new sync
                e_rx_state = RX_IDLE;
            }            
            break;
        case RX_STOP:
            if (uc_byte == END_TOK)
            {   // copy user data
                e_rx_state = RX_READY;
                UART_DISABLE_RX_INT();
            }
            else
            {   // buffer overflow, try new sync
                e_rx_state = RX_IDLE;
            }            
            break;
        default:
            break;
    }
}


// *****************************************************************************
// ***                        Uart_Tx_Empty_Interrupt()                      ***
// *****************************************************************************
// ***  UART transmit register empty interrupt.                              ***
// *****************************************************************************
// ***  Function parameters:                                                 ***
// *****************************************************************************
// ***  Function result:                                                     ***
// *****************************************************************************
ISR(USART_UDRE_vect)
{
  if ( m_tx_len )
  {
    UDR0 = *m_puc_tx_buf++;                                 // send byte
    m_tx_len--;
  }
  if ( !m_tx_len )
  {
    UART_DISABLE_TX_DATA_REG_EMPTY_INT();                   // block IRQ again
  }
}

// *****************************************************************************
// *************************    End UART   *************************************
// *****************************************************************************
